Package: optconerrf
Title: Optimal Monotone Conditional Error Functions
Version: 1.0.3
Authors@R: 
  c(person(given = "Morten", family = "Dreher", email = "morten.dreher@outlook.de", role = c("aut", "cre")),
    person(given = "Werner", family = "Brannath", role = c("aut", "cph"), comment = c(ORCID = "0000-0002-8622-3904")),
    person(given = "Cornelia Ursula", family = "Kunz", role = "ctb", comment = c(ORCID = "0000-0002-8900-9401")),
    person(given = "Johanna", family = "zur Verth", role = "aut"))
Description: Design and analysis of confirmatory adaptive clinical trials using the optimal conditional error framework according to Brannath and Bauer (2004) <doi:10.1111/j.0006-341X.2004.00221.x>. An extension to the optimal conditional error function using interim estimates as described in Brannath and Dreher (2024) <doi:10.48550/arXiv.2402.00814> and functions to ensure that the resulting conditional error function is non-increasing are also available.
Imports: ggplot2, methods
License: GPL (>= 3)
Encoding: UTF-8
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
Config/testthat/edition: 3
URL: https://github.com/morten-dreher/optconerrf
BugReports: https://github.com/morten-dreher/optconerrf/issues
Config/roxygen2/version: 8.1.0
NeedsCompilation: no
Packaged: 2026-09-03 19:45:39 UTC; Morten
Author: Morten Dreher [aut, cre],
  Werner Brannath [aut, cph] (ORCID:
    <https://orcid.org/0000-0002-8622-3904>),
  Cornelia Ursula Kunz [ctb] (ORCID:
    <https://orcid.org/0000-0002-8900-9401>),
  Johanna zur Verth [aut]
Maintainer: Morten Dreher <morten.dreher@outlook.de>
Repository: CRAN
Date/Publication: 2026-09-03 20:10:03 UTC
