# Generated by roxygen2: do not edit by hand

S3method(plot,TrialDesignOptimalConditionalError)
S3method(print,PowerResultsOptimalConditionalError)
S3method(print,SimulationResultsOptimalConditionalError)
S3method(print,TrialDesignOptimalConditionalError)
S3method(summary,TrialDesignOptimalConditionalError)
export(getDesignOptimalConditionalErrorFunction)
export(getExpectedSecondStageInformation)
export(getLikelihoodRatio)
export(getNu)
export(getNuPrime)
export(getOptimalConditionalError)
export(getOverallPower)
export(getPsi)
export(getQ)
export(getSecondStageInformation)
export(getSimulationResults)
importFrom(methods,new)
importFrom(stats,
  pnorm,
  qnorm,
  rnorm
)
